/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/

#ifndef _GALIL1800_H_
#define _GALIL1800_H_

/* Size of read and write hardware fifo */
#define GALIL1800_READ_FIFO_SIZE    512
#define GALIL1800_WRITE_FIFO_SIZE   512

/* Max size of secondary fifo.
   Depend on specific model:
   1840 (152 bytes), 1860 (208 bytes), 1880 (264 bytes) */
#define GALIL1800_MAX_FIFO2_SIZE    264

/* Max len in bytes of serial number
   string read by device (not included '\0') */
#define GALIL1800_SERIAL_MAXLEN 8


#include <linux/ioctl.h>

/* free slot in Documentation/ioctl-number.txt */
#define GALIL1800_IOCTL_MAGIC 'g'

/* ioctl command for "card" device */
#define GALIL1800_IOCTL_RESET_HARDWARE _IO(GALIL1800_IOCTL_MAGIC, 0)
#define GALIL1800_IOCTL_RESET_FIFO_R _IO(GALIL1800_IOCTL_MAGIC, 1)
#define GALIL1800_IOCTL_RESET_FIFO_W _IO(GALIL1800_IOCTL_MAGIC, 2)
#define GALIL1800_IOCTL_RESET_FIFO_RW _IO(GALIL1800_IOCTL_MAGIC, 3)
#define GALIL1800_IOCTL_UNSAFE_WRITE_STOP _IO(GALIL1800_IOCTL_MAGIC, 4)
#define GALIL1800_IOCTL_GET_SERIAL_NUMBER _IOR(GALIL1800_IOCTL_MAGIC, 5, \
					char[GALIL1800_SERIAL_MAXLEN+1])
#define GALIL1800_IOCTL_SET_PAUSE _IOW(GALIL1800_IOCTL_MAGIC, 6, unsigned long)
#define GALIL1800_IOCTL_SET_WRITE_DELAY _IOW(GALIL1800_IOCTL_MAGIC, 7,   \
								unsigned int)
#define GALIL1800_IOCTL_SET_READ_DELAY _IOW(GALIL1800_IOCTL_MAGIC, 8,    \
								unsigned int)
#define GALIL1800_IOCTL_SET_WRITE_TIMEOUT _IOW(GALIL1800_IOCTL_MAGIC, 9, \
								unsigned int)
#define GALIL1800_IOCTL_SET_READ_TIMEOUT _IOW(GALIL1800_IOCTL_MAGIC, 10, \
								unsigned int)

#define GALIL1800_IOCTL_CARD_MINNR 0
#define GALIL1800_IOCTL_CARD_MAXNR 10


/* ioctl command for "event" device */

/* In the following six commands: 1 -> enable event report, 0 -> disable */
#define GALIL1800_IOCTL_EVENT_ALL_ACTIVE _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+1), int)
#define GALIL1800_IOCTL_EVENT_IRQ _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+2), int)
#define GALIL1800_IOCTL_EVENT_WBUFF_LESS_10P _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+3), int)
#define GALIL1800_IOCTL_EVENT_WBUFF_LESS_20P _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+4), int)
#define GALIL1800_IOCTL_EVENT_WBUFF_LESS_50P _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+5), int)
#define GALIL1800_IOCTL_EVENT_WBUFF_OVER_90P _IOW(GALIL1800_IOCTL_MAGIC, \
					(GALIL1800_IOCTL_CARD_MAXNR+6), int)

#define GALIL1800_IOCTL_EVENT_MINNR (GALIL1800_IOCTL_CARD_MAXNR+1)
#define GALIL1800_IOCTL_EVENT_MAXNR (GALIL1800_IOCTL_CARD_MAXNR+6)


/* ioctl command for "info" device */

#define GALIL1800_IOCTL_SET_FIFO2_TIMEOUT _IOW(GALIL1800_IOCTL_MAGIC,   \
				(GALIL1800_IOCTL_EVENT_MAXNR+1), unsigned int)

#define GALIL1800_IOCTL_INFO_MINNR (GALIL1800_IOCTL_EVENT_MAXNR+1)
#define GALIL1800_IOCTL_INFO_MAXNR (GALIL1800_IOCTL_EVENT_MAXNR+1)


/* The type of data returned from the event device
   The driver always returns a number of bytes that
   is a multiple of the size of this data structure */
struct galil1800_event_data {
	unsigned char type; /* type of data info */
	unsigned char data; /* data */
};

/* Values for the type field in the galil1800_event_data structure:
   the first eight values are special, because the corresponding
   events can be disabled by issuing suitable ioctl() commands (refer to
   the opt_flag field in the galil1800_event_buffer structure defined in
   galil1800_event.h */

#define GALIL1800_EVENT_IRQ            0 /* an IRQ was raised */
#define GALIL1800_EVENT_WBUFF_LESS_10P 1 /*write buffer is less than 10% full*/
#define GALIL1800_EVENT_WBUFF_LESS_20P 2 /*write buffer is less than 20% full*/
#define GALIL1800_EVENT_WBUFF_LESS_50P 3 /*write buffer is less than 50% full*/
#define GALIL1800_EVENT_WBUFF_OVER_90P 4 /*write buffer is more than 90% full*/

#define GALIL1800_EVENT_BUFF_OVERWRITTEN    8
/*  the event buffer has overflowed;
    the data field reports the number of
    records lost, or zero if this number
    is more than 255 */

#endif
