/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/

#include "galil1800_io.h"
#include "galil1800_init.h"
#include "galil1800_sysfs.h"

#include <asm/uaccess.h>
#include <linux/poll.h>


static
ssize_t galil1800_info_read(struct file * filp, char __user * buf,
					size_t size, loff_t *poff)
{
	struct galil1800_state *s =
			(struct galil1800_state *) filp->private_data;
	char _buf[GALIL1800_MAX_FIFO2_SIZE];
	ssize_t count;
	ssize_t request = (size < GALIL1800_MAX_FIFO2_SIZE ?
					size : GALIL1800_MAX_FIFO2_SIZE);
	int rc;

	GALIL_PRINT_DEBUG_PID("\"info read\" called\n");

	count = galil1800_read_FIFO2(s, _buf, request);

	if (!count)
		return 0;

	rc = copy_to_user(buf, _buf, count);
	return (rc > 0 ? -EFAULT : count);
}

static
ssize_t galil1800_info_write(struct file * filp, const char __user * buf,
						size_t size, loff_t *poff)
{
	return -EPERM;
}

static
unsigned int galil1800_info_poll(struct file *filp, poll_table *wait)
{
	return -EINVAL;
}

static
int galil1800_info_open(struct inode * inode, struct file * filp)
{
	int ret;
	struct galil1800_state *s;
	struct galil1800_devfile_state *df_s;

	s = container_of(inode->i_cdev, struct galil1800_state,
			devfile_state[GALIL1800_INFO_DEVICE].cdev);
	filp->private_data = s;
	df_s = &(s->devfile_state[GALIL1800_INFO_DEVICE]);

	/* wait for device to become free */

	GALIL_PRINT_DEBUG_PID("\"info open\" started\n");

	if(down_interruptible(&df_s->open_sem)) {
		GALIL_PRINT_DEBUG_PID("\"info open\": \"down\" interrupted\n");
		return -ERESTARTSYS;
	}

	/* Whit this type of check,
	one process can open for read and another for write */

	if(filp->f_mode & FMODE_WRITE) {
		GALIL_PRINT_DEBUG_PID("\"info open\" failed:"
					" FMODE_WRITE is set!\n");
		up(&df_s->open_sem);
		return -EACCES;
	}

	while (df_s->open_mode) {
		if (filp->f_flags & O_NONBLOCK) { /* O_NONBLOCK == O_NDELAY */
			up(&df_s->open_sem);
			GALIL_PRINT_DEBUG_PID("\"info open\" return:"
				" O_NONBLOCK is set and device is busy\n");
			return -EAGAIN;
		}
		up(&df_s->open_sem);
		NO_GALIL_ONE_PRINT_DEBUG_PID("\"info open\" going to sleep\n");
		if(wait_event_interruptible(df_s->open_wait,!df_s->open_mode)) {
			GALIL_PRINT_DEBUG_PID("\"info open\":"
				" \"wait_event\" interrupted\n");
			return -ERESTARTSYS;
		}
		if(down_interruptible(&df_s->open_sem)) {
			GALIL_PRINT_DEBUG_PID("\"info open\":"
					" \"down\" interrupted\n");
			return -ERESTARTSYS;
		}
	}

	ret = galil1800_init_controller_open_step(s);
	if(ret < 0) {
		up(&df_s->open_sem);
		return ret;
	}

	df_s->open_mode = filp->f_mode & FMODE_READ;

	up(&df_s->open_sem);
	nonseekable_open(inode,filp);
	GALIL_PRINT_DEBUG_PID("\"info open\" ended\n");
	return 0;
}

static
int galil1800_info_release(struct inode * inode, struct file * filp)
{
	struct galil1800_state *s =
		(struct galil1800_state *) filp->private_data;
	struct galil1800_devfile_state *df_s =
		&(s->devfile_state[GALIL1800_INFO_DEVICE]);

	GALIL_PRINT_DEBUG_PID("\"info release\" started\n");
	if(down_interruptible(&df_s->open_sem)) {
		GALIL_PRINT_DEBUG_PID("\"info release\":"
				" \"down\" interrupted\n");
		return -ERESTARTSYS;
	}

	df_s->open_mode &= (~filp->f_mode) & FMODE_READ;

	galil1800_clean_controller_open_step(s);

	up(&df_s->open_sem);
	wake_up_interruptible_sync(&df_s->open_wait);
	GALIL_PRINT_DEBUG_PID("\"info release\" ended:"
				" wake up of sleeping open\n");
	return 0;
}

static long galil1800_info_unlocked_ioctl(struct file *filp,
				unsigned int cmd, unsigned long arg)
{
	struct galil1800_state *s =
		(struct galil1800_state *) filp->private_data;

	/* Check command */
	if(_IOC_TYPE(cmd) != GALIL1800_IOCTL_MAGIC)
		return -ENOTTY;
	if(_IOC_NR(cmd) < GALIL1800_IOCTL_INFO_MINNR ||
				_IOC_NR(cmd) > GALIL1800_IOCTL_INFO_MAXNR)
		return -ENOTTY;

	/* Check argument:
	   After this check, we can safely use:
	   __copy_to_user or __copy_from_user
	   __put_user or __get_user          */
	if(_IOC_DIR(cmd) & _IOC_READ) {
		/* Read request, so check if I can write to user space */
		NO_GALIL_PRINT_DEBUG("ioctl info: read request (cmd: %d)\n",
								_IOC_NR(cmd));
		if(!access_ok(VERIFY_WRITE,(void __user *)arg,_IOC_SIZE(cmd)))
			return -EFAULT;
	} else if(_IOC_DIR(cmd) & _IOC_WRITE) {
		/* Write request, so check if I can read from user space */
		NO_GALIL_PRINT_DEBUG("ioctl info: write request (cmd: %d)\n",
								_IOC_NR(cmd));
		if(!access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd)))
			return -EFAULT;
	}

	switch (cmd) {
		case GALIL1800_IOCTL_SET_FIFO2_TIMEOUT:
		{
			unsigned int temp = (unsigned int)arg;
			int ret=galil1800_fifo2_timeout_check_and_convert(&temp);
			s->fifo2_timeout = temp;
			return ret;
		}
		break;
		default:
			return -ENOTTY; /* cmd not valid */
	}

	return 0; /* Success */
}


struct file_operations galil1800_info_fops = {
	.owner = THIS_MODULE,
	.read = galil1800_info_read,
	.write = galil1800_info_write,
	.poll = galil1800_info_poll,
	.open = galil1800_info_open,
	.release = galil1800_info_release,
	.unlocked_ioctl = galil1800_info_unlocked_ioctl,
	.llseek = no_llseek
};

/* Show method for fifo2_timeout class_device_attribute */
static
ssize_t galil1800_sysfs_fifo2_timeout_show(struct class_device *cd, char *buf)
{
	struct galil1800_state *s = container_of(
		(struct galil1800_devfile_state *)(cd->class_data),
		struct galil1800_state, devfile_state[GALIL1800_INFO_DEVICE]);
	return sprintf(buf, "%u\n", (s->fifo2_timeout*1000)/HZ); /* in ms */
}

/* Store method for fifo2_timeout class_device_attribute */
static ssize_t galil1800_sysfs_fifo2_timeout_store(struct class_device *cd,
						const char *buf, size_t count)
{
	unsigned int temp;
	struct galil1800_state *s = container_of(
		(struct galil1800_devfile_state *)(cd->class_data),
		struct galil1800_state, devfile_state[GALIL1800_INFO_DEVICE]);

	GALIL_PRINT_DEBUG("galil1800_sysfs_fifo2_timeout_store, buf: \"%s\"\n",
									 buf);

	if(count > 5 || strlen(buf)!=count) {
		printk(KERN_ERR PFX "fifo2_timeout not modified."
					" Too much characters\n");
		return -EINVAL;
	}

	sscanf(buf, "%u", &temp);

	galil1800_fifo2_timeout_check_and_convert(&temp);

	s->fifo2_timeout = temp;

	return count;
}

/* Add info cdev device and register it in sysfs */
int galil1800_info_cdev_add(struct cdev *cdev_p,
				unsigned int major, unsigned int minor)
{
	int ret;
	struct galil1800_state *s = container_of(cdev_p,
			struct galil1800_state,
			devfile_state[GALIL1800_INFO_DEVICE].cdev);

	cdev_init(cdev_p, &galil1800_info_fops);

	cdev_p->owner = THIS_MODULE;

	ret=cdev_add(cdev_p, MKDEV(major, minor), 1);
	if (ret) {
		printk(KERN_INFO PFX "cannot add info cdev dev=(%d,%d)\n",
								major, minor);
		return ret;
	}

	ret=galil1800_sysfs_class_dev_register(s, GALIL1800_INFO_DEVICE);
	if(ret) {
		printk(KERN_INFO PFX "sysfs class device register"
						" failed for info device\n");
		goto err_class_dev_reg;
	}

	s->cd_attr_fifo2_timeout.attr.name = "fifo2_timeout";
	s->cd_attr_fifo2_timeout.attr.owner = THIS_MODULE;
	s->cd_attr_fifo2_timeout.attr.mode = S_IWUSR | S_IRUGO;
	s->cd_attr_fifo2_timeout.show  = galil1800_sysfs_fifo2_timeout_show;
	s->cd_attr_fifo2_timeout.store = galil1800_sysfs_fifo2_timeout_store;
	ret=class_device_create_file(
		&s->devfile_state[GALIL1800_INFO_DEVICE].class_dev,
		&s->cd_attr_fifo2_timeout);
	if(ret) {
		printk(KERN_ERR PFX "cannot create file for class "
			"device attr \"%s\" in sysfs\n",
			s->cd_attr_fifo2_timeout.attr.name);
		goto err_attr;
	}

	return 0;
err_attr:
	galil1800_sysfs_class_dev_unregister(s, GALIL1800_INFO_DEVICE);
err_class_dev_reg:
	cdev_del(cdev_p);
	return ret;
}

/* Delete info cdev device and deregister it in sysfs */
void galil1800_info_cdev_del(struct cdev *cdev_p)
{
	struct galil1800_state *s = container_of(cdev_p,
			struct galil1800_state,
			devfile_state[GALIL1800_INFO_DEVICE].cdev);

	galil1800_sysfs_class_dev_unregister(s, GALIL1800_INFO_DEVICE);

	class_device_remove_file(
		&s->devfile_state[GALIL1800_INFO_DEVICE].class_dev,
		&s->cd_attr_fifo2_timeout );

	cdev_del(cdev_p);
}
